/*
 * DataConsumer.java
 *
 * Created on 15. kvten 2005, 22:42
 */

package cz.matfyz.sykora.nngui.data;

import cz.matfyz.sykora.nngui.exception.*;

/**
 * Interface pro abstraktnho konzumenta dat z neuronov st. Slou pro
 * pedvn vstupu neuronov st k dalmu zpracovn, nebo k zpisu do
 * souboru
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public interface DataConsumer {
    /**
     * Uzaven vstupnho souboru. Je volna potom, co je potvrzen posledn
     * zznam pedan konzumentovi
     */
    public void close();
    /**
     * Potvrzen zadan dky pro dal zpracovn. Dokued nen zavolna tato
     * funkce, nepovauje konzument rdku za sprvn zadanou a nepracuje s n.
     *  @return false pokud konzument ji nepijm dal data, jinak true
     */
    public boolean commitRow();
    /**
     * Zjitn velikosti dky pijman konzumentem. Vrac velikost nastavenou
     * pomoc <i>setRowSize</i>
     *  @return poet (oekvanch) hodnot v jednom zznamu
     *  @see #setRowSize
     */
    public int getRowSize();
    /**
     * Nastaven velikosti dky ve vstupnm souboru. Funkce mus bt zavolna
     * ped zapsnm prvn hodnoty do vstupnho souboru.
     *  @param _size poet hodnot v zznamu ve vstupnm souboru
     *  @return true pokud je nastavovan poet hodnot pijateln, jinak false
     */
    public boolean setRowSize(int _size);
    /**
     * Ped jednu hodnotu z aktulnho zaznamu konzumentovi. Hodnoty, kter
     * pro zpracovvan zznam nebyly nastaveny pomoc setValue nejsou
     * definovny a ve vstupnm souboru se na jejich mst me objevit
     * libovoln slo
     *  @param _index index hodnoty v zznamu. Index prvn hodnoty je 0, index
     *              posledn je getRowSize() - 1
     *  @param _value pedvan hodnota
     */
    public void setValue(int _index, double _value) throws DataException;
}
