/*
 * MainFrame.java
 *
 * Created on 4. kvten 2005, 14:39
 */

package cz.matfyz.sykora.nngui.gui;

import cz.matfyz.sykora.nngui.exception.*;
import cz.matfyz.sykora.nngui.network.*;
import cz.matfyz.sykora.nngui.data.*;

import java.awt.*;
import java.io.*;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * Hlavn okno programu. Pomoc menu lze vykonvat vechny potebn operace s
 * neuronovou st. Obsahuje tak vstupn bod programu.
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class MainFrame extends javax.swing.JFrame {
    /**
     * Aktuln stav prv zpracovvan st.
     */
    Network             currentNetwork;
    /**
     * Jmno souboru ve kterm je uloena prv zpracovvan s, resp. null
     * pokud s byla vytvoen pomoc pkazu <i>New network</i> z hlavnho
     * menu.
     *  @see #onNewNetwork
     *  @see #onLoadNetwork
     *  @see #onSaveNetwork
     *  @see #onSaveNetworkAs
     */
    String              currentNetworkFileName;
    /**
     * Stavitel pouvan pro natn a ukldn st.
     *  @see #currentNetwork
     */
    NetworkBuilder      networkBuilder;
    /**
     * Komponenta pro obsluhu vybrn soubor pro natn/ukldn struktury
     * neuronov st.
     *  @see #initFileChooser
     *  @see #onLoadNetwork
     *  @see #onSaveNetworkAs
     */
    JFileChooser        networkFileChooser;
    
    /**
     * Konstruktor. Initializuje ovldac prvky.
     */
    public MainFrame() {
        currentNetwork = null;
        networkBuilder = new NetworkBuilder();
        
        // inicializace komponent site
        initComponents();
        initFileChooser();
        // disabluj polozky menu, kdyz neni vytvorena zadna sit a neni tedy
        // s cim pocitat
        enableMenuItems(false);
        
        outputLog.append("Application started...\n");
    }
    
    /**
     * Povolen/zzkaz ovldacch prvk v menu. Povoluje/zakazuje ty ovldac
     * prvky, kter je mon pout jen kdy u byla vytvoena njak neuronov
     * s.
     *  @param _enable pepn mezi povolenm/zkazem ovldacch prvk
     *  @see #MainFrame
     *  @see #onNewNetwork
     *  @see #onLoadNetwork
     */
    private void enableMenuItems(boolean _enable) {
        mainMenuNetworkSave.setEnabled(_enable);
        mainMenuNetworkSaveAs.setEnabled(_enable);
        mainMenuDataProcessFile.setEnabled(_enable);
        mainMenuDataProcessGUI.setEnabled(_enable);
        mainMenuDataTrain.setEnabled(_enable);
        mainMenuArchitectureEdit.setEnabled(_enable);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        javax.swing.JSeparator jSeparator1;
        javax.swing.JSeparator jSeparator2;
        javax.swing.JMenu mainMenuData;
        javax.swing.JMenu mainMenuDataProcess;

        jPanel1 = new javax.swing.JPanel();
        outputLog = new javax.swing.JTextArea();
        mainMenu = new javax.swing.JMenuBar();
        mainMenuNetwork = new javax.swing.JMenu();
        mainMenuNetworkNew = new javax.swing.JMenuItem();
        mainMenuNetworkLoad = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        mainMenuNetworkSave = new javax.swing.JMenuItem();
        mainMenuNetworkSaveAs = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        mainMenuNetworkExit = new javax.swing.JMenuItem();
        mainMenuData = new javax.swing.JMenu();
        mainMenuDataProcess = new javax.swing.JMenu();
        mainMenuDataProcessFile = new javax.swing.JMenuItem();
        mainMenuDataProcessGUI = new javax.swing.JMenuItem();
        mainMenuDataTrain = new javax.swing.JMenuItem();
        mainMenuArchitecture = new javax.swing.JMenu();
        mainMenuArchitectureEdit = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("BackProp Studio");
        outputLog.setEditable(false);
        outputLog.setPreferredSize(new java.awt.Dimension(300, 200));
        getContentPane().add(outputLog, java.awt.BorderLayout.CENTER);

        mainMenuNetwork.setText("Network");
        mainMenuNetworkNew.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        mainMenuNetworkNew.setText("New...");
        mainMenuNetworkNew.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onNewNetwork(evt);
            }
        });

        mainMenuNetwork.add(mainMenuNetworkNew);

        mainMenuNetworkLoad.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        mainMenuNetworkLoad.setText("Open...");
        mainMenuNetworkLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onLoadNetwork(evt);
            }
        });

        mainMenuNetwork.add(mainMenuNetworkLoad);

        mainMenuNetwork.add(jSeparator1);

        mainMenuNetworkSave.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        mainMenuNetworkSave.setText("Save");
        mainMenuNetworkSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSaveNetwork(evt);
            }
        });

        mainMenuNetwork.add(mainMenuNetworkSave);

        mainMenuNetworkSaveAs.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        mainMenuNetworkSaveAs.setText("Save as...");
        mainMenuNetworkSaveAs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSaveNetworkAs(evt);
            }
        });

        mainMenuNetwork.add(mainMenuNetworkSaveAs);

        mainMenuNetwork.add(jSeparator2);

        mainMenuNetworkExit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, java.awt.event.InputEvent.ALT_MASK));
        mainMenuNetworkExit.setText("Exit");
        mainMenuNetworkExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onExit(evt);
            }
        });

        mainMenuNetwork.add(mainMenuNetworkExit);

        mainMenu.add(mainMenuNetwork);

        mainMenuData.setText("Data");
        mainMenuDataProcess.setText("Process");
        mainMenuDataProcessFile.setText("From file...");
        mainMenuDataProcessFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onProcessDataFromFile(evt);
            }
        });

        mainMenuDataProcess.add(mainMenuDataProcessFile);

        mainMenuDataProcessGUI.setText("From GUI...");
        mainMenuDataProcessGUI.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onProcessDataFromGUI(evt);
            }
        });

        mainMenuDataProcess.add(mainMenuDataProcessGUI);

        mainMenuData.add(mainMenuDataProcess);

        mainMenuDataTrain.setText("Train network...");
        mainMenuDataTrain.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onTrainNetwork(evt);
            }
        });

        mainMenuData.add(mainMenuDataTrain);

        mainMenu.add(mainMenuData);

        mainMenuArchitecture.setText("Architecture");
        mainMenuArchitectureEdit.setText("Edit...");
        mainMenuArchitectureEdit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onEditArchitecture(evt);
            }
        });

        mainMenuArchitecture.add(mainMenuArchitectureEdit);

        mainMenu.add(mainMenuArchitecture);

        setJMenuBar(mainMenu);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents

    private void onProcessDataFromGUI(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onProcessDataFromGUI
        try {
            ProcessDataInteractiveDialog dlg = new ProcessDataInteractiveDialog(this, currentNetwork);
            dlg.setVisible(true);
        }
        catch(Exception e) {
            
        }
    }//GEN-LAST:event_onProcessDataFromGUI

    /**
     * Uloen struktury neuronov st do zadanho souboru. Vytv si pro
     * potebu ukldn DOM implementaci a dokument
     *  @param _file_name jmno souboru, do kterho se ukld
     *  @see NetworkBuilder#exportNetworkToXML
     *  @see #onSaveNetwork
     *  @see #onSaveNetworkAs
     */
    private void saveNetwork(String _file_name) {
        try {
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder doc_builder = factory.newDocumentBuilder();
            
            // exportuj sit do DOM dokuemntu
            Document doc = doc_builder.newDocument();
            Node network_node = networkBuilder.exportNetworkToXML(doc, currentNetwork);
            
            // proved transformaci DOM dokumentu do textove podoby
            // TODO: mohlo by se hodit "pekne" formatovani textove podoby XML
            //          dokumentu
            TransformerFactory trans_factory = TransformerFactory.newInstance();
            Transformer transformer = trans_factory.newTransformer();
            transformer.transform(new DOMSource(network_node), new StreamResult(networkFileChooser.getSelectedFile()));
        }
        catch(FactoryConfigurationError f_err) {
            // doslo k nejake chybe pri vytvareni DocumentBuilderFactory
            JOptionPane.showMessageDialog(this, "Coult not create DOM document factory for building the document representation: " + f_err.getLocalizedMessage(),
                            "Saving network to file", JOptionPane.ERROR_MESSAGE);
        }
        catch(ParserConfigurationException p_err) {
            // doslo k nejake chybe pri vytvareni DocumentBuilder
            JOptionPane.showMessageDialog(this, "COuld not create DOM document to save the network structure: " + p_err.getLocalizedMessage(),
                            "Saving network to file", JOptionPane.ERROR_MESSAGE);
        }
        catch(Exception e) {
            // proste nejaka jina chyba (mozna DOMException)
            JOptionPane.showMessageDialog(this, "An unexpected exception occured while saving the network to file: " + e.getLocalizedMessage(),
                            "Saving network to file", JOptionPane.ERROR_MESSAGE);
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    }
    
    /**
     * Obsluha poloky menu <i>Network/Save</i>. Ukld s do souboru
     * <i>currentNetworkFileName</i>, pokud nen zadno jmno souboru (tj.
     * ukldan s byla nov vytvoena a jet nebyla uloen), pepne se do
     * Save As.. mdu.
     *  @param evt informace o udlosti v GUI
     *  @see #currentNetworkFileName
     *  @see #onSaveNetworkAs
     *  @see #saveNetwork
     */
    private void onSaveNetwork(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onSaveNetwork
        assert(currentNetwork != null);
        if(currentNetworkFileName == null)
            onSaveNetworkAs(evt);
        saveNetwork(currentNetworkFileName);
    }//GEN-LAST:event_onSaveNetwork

    /**
     * Obsluha poloky v menu <i>Data/Train Network...</i>.
     * Vytv novou instanci dialogu pro trnovn neuronov st
     *  @param evt informace o udlosti v GUI
     *  @see TrainNetworkDialog
     */
    private void onTrainNetwork(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onTrainNetwork
        TrainNetworkDialog dlg = new TrainNetworkDialog(currentNetwork, this);
        dlg.setVisible(true);
    }//GEN-LAST:event_onTrainNetwork

    /**
     * Obsluha poloky v menu <i>Network/Save As...</i>. Nech uivatele vybrat
     * jmno souboru, do kterho bude s uloena.
     *  @param evt informace o udlosti v GUI
     *  @see #currentNetworkFileName
     *  @see #networkFileChooser
     *  @see #saveNetwork
     */
    private void onSaveNetworkAs(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onSaveNetworkAs
        int ret = networkFileChooser.showSaveDialog(this);
        if(JFileChooser.APPROVE_OPTION != ret)
            return;
        currentNetworkFileName = networkFileChooser.getSelectedFile().getAbsolutePath();
        saveNetwork(currentNetworkFileName);
    }//GEN-LAST:event_onSaveNetworkAs

    /**
     * Inicializuje komponentu zajiujc vbr jmna souboru pro natn
     * a ukldn struktury st.
     *  @see #networkFileChooser
     */
    private void initFileChooser() {
        networkFileChooser = new JFileChooser();
        networkFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    }
    
    /**
     * Obsluha poloky v menu <i>Network/Load...</i>. Nech uivatele vybrat
     * soubor s neuronovou st a tu z nj potom nate.
     *  @param evt informace o udlosti v GUI
     *  @see NetworkBuilder#createNetwork(org.w3c.dom.Node)
     */
    private void onLoadNetwork(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onLoadNetwork
        int ret = networkFileChooser.showOpenDialog(this);
        if(JFileChooser.APPROVE_OPTION != ret)
            return;
        
        File selected_file = networkFileChooser.getSelectedFile();
        if(!selected_file.canRead()) {
            JOptionPane.showMessageDialog(this, "File does not exist or cant't be read");
            return;
        }
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder doc_builder = factory.newDocumentBuilder();
            
            Document doc = doc_builder.parse(selected_file);
            currentNetwork = networkBuilder.createNetwork(doc.getDocumentElement());
            enableMenuItems(true);
        }
        catch(FactoryConfigurationError f_err) {
            JOptionPane.showMessageDialog(this, "Could not create DOM document builder factory to be user while parsing the document: " + f_err.getLocalizedMessage(),
                            "Loading network from file", JOptionPane.ERROR_MESSAGE);
        }
        catch(ParserConfigurationException p_err) {
            JOptionPane.showMessageDialog(this, "Could not create DOM document builder to be used for parsing the document: " + p_err.getLocalizedMessage(),
                            "Loading network from file", JOptionPane.ERROR_MESSAGE);
        }
        catch(SAXException s_err) {
            JOptionPane.showMessageDialog(this, "There was an error while parsing the XML file: " + s_err.getLocalizedMessage(),
                            "Loading network from file", JOptionPane.ERROR_MESSAGE);
        }
        catch(InvalidDataFormatException df_err) {
            JOptionPane.showMessageDialog(this, "There was an error while creating the network from XML representation: " + df_err.getLocalizedMessage(),
                            "Loading network from file", JOptionPane.ERROR_MESSAGE);
        }
        catch(UnsupportedNetworkModelException nm_err) {
            JOptionPane.showMessageDialog(this, "The loaded network uses model that is not supported: " + nm_err.getLocalizedMessage(),
                            "Loading network from file", JOptionPane.ERROR_MESSAGE);
        }
        catch(UnsupportedNetworkTypeException nt_err) {
            JOptionPane.showMessageDialog(this, "The loaded network is of unsupported type: " + nt_err.getLocalizedMessage(),
                            "Loading network from file", JOptionPane.ERROR_MESSAGE);
        }
        catch(IOException io_err) {
            JOptionPane.showMessageDialog(this, "Unexpected I/O error occured: " + io_err.getLocalizedMessage(),
                            "Loading network from file", JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_onLoadNetwork

    /**
     * Obsluha poloky menu <i>Data/Process/From File...</i>. Inicializuje a
     * zobraz dialog pro zpracovn dat.
     *  @param evt informace o udlosti v GUI
     */
    private void onProcessDataFromFile(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onProcessDataFromFile
        ProcessDataFileDialog dlg = new ProcessDataFileDialog(currentNetwork, this);
        dlg.setVisible(true);
    }//GEN-LAST:event_onProcessDataFromFile

    /**
     * Obsluha poloky menu <i>Network/New...</i>. Zobraz dialog pro editaci
     * architektury st a na zklad dat z nj vytvo novou s.
     *  @param evt informace o udlosti v GUI
     */
    private void onNewNetwork(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onNewNetwork
        ArchitectureEditorDialog arch_dlg = new ArchitectureEditorDialog(this);
        arch_dlg.setVisible(true);
        
        int[] new_architecture = arch_dlg.exportData();
        currentNetwork = networkBuilder.createNetwork(new_architecture, new SigmoidModel());
        currentNetworkFileName = null;
        
        enableMenuItems(true);
    }//GEN-LAST:event_onNewNetwork

    /**
     * Obsluha poloky menu <i>Architecture/Edit...</i>. Zobraz dialog pro
     * editaci architekruy st a pak s nm nic nedl.
     *  @param evt informace o udlosti v GUI
     */
    private void onEditArchitecture(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onEditArchitecture
        assert(currentNetwork != null);
        
        ArchitectureEditorDialog dlg = new ArchitectureEditorDialog(this);
        int[] old_architecture = currentNetwork.getArchitecture();
        
        dlg.initializeData(old_architecture);
        dlg.setVisible(true);
        
        int[] new_architecture = dlg.exportData();
        boolean changed = old_architecture.length != new_architecture.length;
        if(!changed) {
            for(int i = 0; i < old_architecture.length; i++)
                if(old_architecture[i] != new_architecture[i]) {
                    changed = true;
                    break;
                }
        }
        
        if(changed)
            currentNetwork = networkBuilder.createNetwork(new_architecture, new SigmoidModel());
    }//GEN-LAST:event_onEditArchitecture

    /**
     * Obsluha poloky menu <i>Network/Exit</i>. Bez diskus zave hlavn okno
     * a tm ukon program.
     *  @param evt informace o udlosti v GUI
     */
    private void onExit(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onExit
        this.dispose();
    }//GEN-LAST:event_onExit
    
    /**
     * Hlavn funkce programu. Vytvo hlavn okno programu a ped mu zen.
     *  @param args parametry pkazov dky, keter jsou velkoryse ignorovny
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new MainFrame().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JMenuBar mainMenu;
    private javax.swing.JMenu mainMenuArchitecture;
    private javax.swing.JMenuItem mainMenuArchitectureEdit;
    private javax.swing.JMenuItem mainMenuDataProcessFile;
    private javax.swing.JMenuItem mainMenuDataProcessGUI;
    private javax.swing.JMenuItem mainMenuDataTrain;
    private javax.swing.JMenu mainMenuNetwork;
    private javax.swing.JMenuItem mainMenuNetworkExit;
    private javax.swing.JMenuItem mainMenuNetworkLoad;
    private javax.swing.JMenuItem mainMenuNetworkNew;
    private javax.swing.JMenuItem mainMenuNetworkSave;
    private javax.swing.JMenuItem mainMenuNetworkSaveAs;
    private javax.swing.JTextArea outputLog;
    // End of variables declaration//GEN-END:variables
    
}
