/*
 * ProcessDataFileDialog.java
 *
 * Created on 23. ervenec 2005, 10:59
 */

package cz.matfyz.sykora.nngui.gui;

import cz.matfyz.sykora.nngui.data.*;
import cz.matfyz.sykora.nngui.exception.DataException;
import cz.matfyz.sykora.nngui.network.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;

/**
 * Dialog pro zpracovn dat ze souboru s vstupem do jinho souboru.
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class ProcessDataFileDialog extends javax.swing.JDialog {
    /**
     * S pomoc kter jsou zpracovvny data. Oekv se, e je ji nauen,
     * aby dvala njak smyslupln vsledky. Bhem vpotu je sti nastaven
     * nov provider dat a jejich konzument, po pouit st v tomto dialogu
     * nelze nijak spolhat na nastaven jejho konzumenta/producenta dat.
     *  @see #inputFile
     *  @see #onStartProcessing
     *  @see #outputFile
     */
    Network network;
    /**
     * Provider, kter te data ze vstupnho souboru a pedkld je sti.
     *  @see #onStartProcessing
     */
    DataProvider inputFile;
    /**
     * Konzument, kter ukld data produkovan st do vstupnho souboru
     *  @see #onStartProcessing
     */
    DataConsumer outputFile;
    /**
     * Komponenta zajiujc dialog pro vbr vstupnho a vstupnho souboru
     *  @see #onSelectSourceFile
     *  @see #onSelectTargetFile
     */
    JFileChooser fileChooser;
    /**
     * Promnn ovlivnujc, jestli vpoet probh. Jejm nastavenm na false
     * je mon (pomoc tlatka <i>Stop</i>) zastavit.
     *  @see #onStartProcessing
     *  @see #onStopProcessing
     */
    volatile boolean processingActive;
    
    /** 
     * Konstruktor. Inicializuje dialog pro zadanou s.
     *  @param _network s, pro kterou dialog inicializujeme
     *  @param _parent rodi tohoto okna
     */
    public ProcessDataFileDialog(Network _network, java.awt.Frame _parent) {
        super(_parent, true);
        initComponents();
        
        network = _network;
        assert(null != network);
        
        fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setMultiSelectionEnabled(false);
        
        enableControls(true);
    }
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        javax.swing.JLabel sourceFileLabel;
        javax.swing.JLabel targetFileLabel;

        sourceFileLabel = new javax.swing.JLabel();
        sourceFileEditor = new javax.swing.JTextField();
        selectSourceFileButton = new javax.swing.JButton();
        targetFileLabel = new javax.swing.JLabel();
        targetFileEditor = new javax.swing.JTextField();
        selectTargetFileButton = new javax.swing.JButton();
        progressLabel = new javax.swing.JLabel();
        progressReportLabel = new javax.swing.JLabel();
        stopButton = new javax.swing.JButton();
        startButton = new javax.swing.JButton();

        getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Process data from file");
        setName("processFileDlg");
        setResizable(false);
        sourceFileLabel.setText("Source file:");
        getContentPane().add(sourceFileLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, 70, 20));

        sourceFileEditor.setMaximumSize(new java.awt.Dimension(2147483647, 20));
        getContentPane().add(sourceFileEditor, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 10, 220, -1));

        selectSourceFileButton.setText("...");
        selectSourceFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSelectSourceFile(evt);
            }
        });

        getContentPane().add(selectSourceFileButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 10, 20, 20));

        targetFileLabel.setText("Target file:");
        getContentPane().add(targetFileLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, 70, 20));

        targetFileEditor.setMaximumSize(new java.awt.Dimension(2147483647, 20));
        getContentPane().add(targetFileEditor, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 40, 220, -1));

        selectTargetFileButton.setText("...");
        selectTargetFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSelectTargetFile(evt);
            }
        });

        getContentPane().add(selectTargetFileButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 40, 20, 20));

        progressLabel.setText("Progress");
        getContentPane().add(progressLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, -1, -1));

        progressReportLabel.setText("(not started yet)");
        progressReportLabel.setBorder(new javax.swing.border.EtchedBorder());
        getContentPane().add(progressReportLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 80, 250, -1));

        stopButton.setText("Stop");
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onStopProcessing(evt);
            }
        });

        getContentPane().add(stopButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 110, 70, -1));

        startButton.setText("Start");
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onStartProcessing(evt);
            }
        });

        getContentPane().add(startButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(190, 110, 70, -1));

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-363)/2, (screenSize.height-181)/2, 363, 181);
    }
    // </editor-fold>//GEN-END:initComponents

    /**
     * Povolen/zkaz ovldacch prvk v tomto dialogu. Umouje bu povolit ve
     * mimo tlatka <i>Stop</i> (mimo vpoet), nebo zakzat ve mimo tlatka
     * <i>Stop</i> a znemonit zaven okna (po dobu vpotu).
     *  @param _enable pokud je true, jsou ovldac prvky (s vjimkou
     *              <i>Stop</i> povoleny, jinak jsou zakzny
     */
    private void enableControls(boolean _enable) {
        selectSourceFileButton.setEnabled(_enable);
        selectTargetFileButton.setEnabled(_enable);
        sourceFileEditor.setEnabled(_enable);
        targetFileEditor.setEnabled(_enable);
        startButton.setEnabled(_enable);
        stopButton.setEnabled(!_enable);
        setDefaultCloseOperation(_enable ? JFrame.DISPOSE_ON_CLOSE : JFrame.DO_NOTHING_ON_CLOSE);
    }
    
    /**
     * Zastaven vpotu (nastav <i>processingActive</i> na false, m poru
     * podmnku ve while cyklu ve vlknu v <i>onStartProcessing</i>.
     *  @param evt informace o udlosti v GUI
     *  @see #onStartProcessing
     *  @see #processingActive
     */
    private void onStopProcessing(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onStopProcessing
        processingActive = false;
    }//GEN-LAST:event_onStopProcessing

    /**
     * Obsluha tlatka <i>Start</i> z GUI. Provede inicializaci providera a
     * konzumenta dat, jejich propojen se st a spus nov vlnko, ve kterm
     * b samotn vpoet.
     *  @param evt informace o udlosti v GUI
     *  @see #inputFile
     *  @see #network
     *  @see #onStopProcessing
     *  @see #outputFile
     *  @see #processingActive
     */
    private void onStartProcessing(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onStartProcessing
        if(targetFileEditor.getText().length() == 0) {
            JOptionPane.showMessageDialog(this, "No source file was specified",
                    "File processing", JOptionPane.WARNING_MESSAGE);
            return;
        }
        if(sourceFileEditor.getText().length() == 0) {
            JOptionPane.showMessageDialog(this, "No target file was specified",
                    "File processing", JOptionPane.WARNING_MESSAGE);
            return;
        }
        
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        try {
            File input = new File(sourceFileEditor.getText());
            if(!input.exists())
                throw new FileNotFoundException("Source file does not exist");
            inputFile = new TextStreamProvider(sourceFileEditor.getText());
        }
        catch(Exception excp) {
            JOptionPane.showMessageDialog(this, "Source file does not exist or could not be open: " + excp.getLocalizedMessage(),
                        "File processing", JOptionPane.ERROR_MESSAGE);
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        
        try {
            File output = new File(targetFileEditor.getText());
            if(output.exists() && !output.canWrite())
                throw new Exception("Target file is not writable");
            outputFile = new TextStreamConsumer(targetFileEditor.getText());
        }
        catch(Exception excp) {
            JOptionPane.showMessageDialog(this, "Target file could not be open: " + excp.getLocalizedMessage(),
                        "File processing", JOptionPane.ERROR_MESSAGE);
            inputFile.close();
            inputFile = null;
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        
        try {
            network.setInputProvider(inputFile);
        }
        catch(DataException data) {
            String message = "Sizes of input layer (" + network.getInputRowSize() +
                        ") and input file record (" + inputFile.getRowSize() + ") do " +
                        "not match, data could not be processed.";
            JOptionPane.showMessageDialog(this, message, "File processig", JOptionPane.ERROR_MESSAGE);
            inputFile.close();
            inputFile = null;
            outputFile.close();
            outputFile = null;
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        try {
            network.setOutputConsumer(outputFile);
        }
        catch(DataException data) {
            String message = "Output file does not accept records of size " + network.getOutputRowSize();
            JOptionPane.showMessageDialog(this, message, "File processing", JOptionPane.ERROR_MESSAGE);
            inputFile.close();
            inputFile = null;
            outputFile.close();
            outputFile = null;
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        
        final JDialog parent = this;
        Thread working = new Thread() {
            public void run() {
                try {
                    int records = 0;
                    processingActive = true;
                    enableControls(false);
                    while(processingActive && network.calculateSingle()) {
                        progressReportLabel.setText(Integer.toString(records) + " record(s) processed");
                        records++;
                    }
                }
                catch(Exception data) {
                    String message = "Data processing failed with an exception: " + data.getLocalizedMessage();
                    JOptionPane.showMessageDialog(parent, message, "File processing", JOptionPane.ERROR_MESSAGE);
                }
                finally {
                    inputFile.close();
                    inputFile = null;
                    outputFile.close();
                    outputFile = null;
                    try {
                        network.setOutputConsumer(null);
                    }
                    catch(DataException d_err) {

                    }
                    enableControls(true);
                    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                }                
            }
        };
        working.start();
    }//GEN-LAST:event_onStartProcessing

    /**
     * Obsluha tlatka pro vbr clovho souboru.
     *  @param evt informace o udlosti v GUI
     */
    private void onSelectTargetFile(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onSelectTargetFile
        int res = fileChooser.showOpenDialog(this);
        if(JFileChooser.APPROVE_OPTION != res)
            return;
        targetFileEditor.setText(fileChooser.getSelectedFile().getAbsolutePath());
    }//GEN-LAST:event_onSelectTargetFile

    /**
     * Obsluha tlatka pro vbr zdrojovho souboru.
     *  @param evt informace o udlosti v GUI
     */
    private void onSelectSourceFile(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onSelectSourceFile
        int res = fileChooser.showOpenDialog(this);
        if(JFileChooser.APPROVE_OPTION != res || !fileChooser.getSelectedFile().exists())
            return;
        sourceFileEditor.setText(fileChooser.getSelectedFile().getAbsolutePath());
    }//GEN-LAST:event_onSelectSourceFile
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel progressLabel;
    private javax.swing.JLabel progressReportLabel;
    private javax.swing.JButton selectSourceFileButton;
    private javax.swing.JButton selectTargetFileButton;
    private javax.swing.JTextField sourceFileEditor;
    private javax.swing.JButton startButton;
    private javax.swing.JButton stopButton;
    private javax.swing.JTextField targetFileEditor;
    // End of variables declaration//GEN-END:variables
    
}
