/*
 * ProcessDataInteractiveDialog.java
 *
 * Created on 27. ervenec 2005, 21:12
 */

package cz.matfyz.sykora.nngui.gui;

import cz.matfyz.sykora.nngui.data.*;
import cz.matfyz.sykora.nngui.exception.*;
import cz.matfyz.sykora.nngui.network.*;
import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;

/**
 * Dialog umoujc interaktivn zadvat data a pebrat vsledky vpotu
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class ProcessDataInteractiveDialog extends javax.swing.JDialog
                                          implements DataProvider, DataConsumer {
    
    /**
     * Rozen tdy DefaultTableModel zjednoduujc zkladn koly pi prci
     * s modelem st.
     */
    private static class RowTableModel extends DefaultTableModel {
        
        /**
         * Zjitn tdy sloupce. Vechny sloupce v tabulce jsou typu Double
         *  @param _column index dotazovanho sloupeku
         */
        public Class getColumnClass(int _column) {
            return Double.class;
        }
        
        /**
         * Konstruktor. Inicializuje tabulku s jednou dkou a zadanm poem
         * sloupc.
         *  @param _size poet sloupc pro kter m bt tabulka inicializovna
         */
        public RowTableModel(int _size) {
            super(1, _size);
            
            for(int i = 0; i < _size; i++)
                setValueAt(new Double(0.0), 0, i);
        }
    }
    
    /**
     * S pomoc kter jsou potna data. Bhem konstrukce tohoto dialogu je
     * tento dialog sti nastaven jako konzument i jako producent dat, po
     * ukonen dialogu je nastaven konzumenta a producenta dat nedefinovan
     * a ped dalm vpotem st musej bt znovu nastaveny.
     *  @see #onCalculate
     */
    private Network network;
    
    /**
     * Konstruktor. Inicializuje dialog pro zadanou neuronovou s.
     *  @param _paren rodiovsk okno pro tento dialog
     *  @param _network s, kterou m dialog pouivat bhem vptu
     */
    public ProcessDataInteractiveDialog(java.awt.Frame _parent, Network _network) throws Exception {
        super(_parent, true);
        initComponents();
        
        inputDataTable.setModel(new RowTableModel(_network.getInputRowSize()));
        outputDataTable.setModel(new RowTableModel(_network.getOutputRowSize()));
                
        network = _network;
        try {
            network.setInputProvider(this);
            network.setOutputConsumer(this);
        }
        catch(Exception e) {
            JOptionPane.showMessageDialog(this, "Unexpected exception occured: " + e.getLocalizedMessage(),
                        "Interactive data processing", JOptionPane.ERROR_MESSAGE);
            throw e;
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        javax.swing.JLabel labelInputData;
        javax.swing.JLabel labelOutputData;

        labelInputData = new javax.swing.JLabel();
        inputScrollPane = new javax.swing.JScrollPane();
        inputDataTable = new javax.swing.JTable();
        labelOutputData = new javax.swing.JLabel();
        outputScrollPane = new javax.swing.JScrollPane();
        outputDataTable = new javax.swing.JTable();
        calculateButton = new javax.swing.JButton();

        getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.Y_AXIS));

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        labelInputData.setText("Input data");
        getContentPane().add(labelInputData);

        inputDataTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        inputScrollPane.setViewportView(inputDataTable);

        getContentPane().add(inputScrollPane);

        labelOutputData.setText("Output data");
        getContentPane().add(labelOutputData);

        outputDataTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        outputScrollPane.setViewportView(outputDataTable);

        getContentPane().add(outputScrollPane);

        calculateButton.setText("Calculate");
        calculateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onCalculate(evt);
            }
        });

        getContentPane().add(calculateButton);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents

    /**
     * Obsluha tlatka <i>Calculate</i>. Provede jeden krok vpotu st.
     *  @param evt informace o udlosti v GUI
     */
    private void onCalculate(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onCalculate
        try {
            network.calculateSingle();
        }
        catch(Exception e) {
            
        }
    }//GEN-LAST:event_onCalculate
    
    /**
     * Uzaven providera/konzumenta. Ani jednoho z nich nen poteba v tomto
     * ppad uzavrat, proto tato metoda nedl nic.
     */
    public void close() {
        
    }
    
    /**
     * Ukonen pedvn dky konzumentu. Vdy uspje, data jsou zapisovna
     * pmo do tabulky, proto tato metoda jen potvrd spch.
     *  @return vdy true
     */
    public boolean commitRow() {
        return true;
    }
    
    /**
     * Naten zznamu providera. Data jsou tena pmo z tabulky, ta navc
     * obsahuje vdy platn data, proto tato metoda jen potvrd spch
     *  @return vdy true
     */
    public boolean fetchRow() {
        return true;
    }
    
    /**
     * Zjitn velikosti zznamu providera dat. Odvozuje ho z velikosti vstupn
     * tabulky.
     *  @return poet sloupc ve vstupn tabulce
     */
    public int getRowSize() {
        return inputDataTable.getColumnCount();
    }
    
    /**
     * Peten vstupn hodnoty na zadan pozici z providera.
     *  @param _index pozice ten hodnoty
     *  @return hodnota na pozici <i>_index</i>
     */
    public double getValue(int _index) {
        if(_index < 0 || _index >= inputDataTable.getColumnCount())
            throw new IndexOutOfBoundsException();
      
        return ((Number)inputDataTable.getValueAt(0, _index)).doubleValue();
    }
    
    /**
     * Petoen vstupnch vzork zpt na zatek. Protoe zde nen zatek ani
     * konec vstupnho souboru dat, vdy uspje.
     *  @return vdy true
     */
    public boolean rewind() {
        return true;
    }
    
    /**
     * Nastaven velikosti zznamu konzumenta. Pouze kontroluje, jestli pedan
     * velikost odpovd skuten velikosti vstupn vrstvy st, pro kterou je
     * vstupn tabulka inicializovna.
     *  @param _size velikost zznamu konzumenta
     *  @return true pokud zadan velikost odpovd velikosti vstupn vrstvy
     *              st, jinak false
     */
    public boolean setRowSize(int _size) {
        return _size == outputDataTable.getColumnCount();
    }
    
    /**
     * Pedn hodnoty na zadan pozici konzumentovi
     *  @param _index pozice pedvan hodnoty
     *  @param _value pedvan hodnota
     */
    public void setValue(int _index, double _value) {
        if(_index < 0 || _index >= outputDataTable.getColumnCount())
            throw new IndexOutOfBoundsException();
        
        outputDataTable.setValueAt(new Double(_value), 0, _index);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton calculateButton;
    private javax.swing.JTable inputDataTable;
    private javax.swing.JScrollPane inputScrollPane;
    private javax.swing.JTable outputDataTable;
    private javax.swing.JScrollPane outputScrollPane;
    // End of variables declaration//GEN-END:variables
    
}
