/*
 * TrainNetworkDialog.java
 *
 * Created on 23. ervenec 2005, 20:06
 */

package cz.matfyz.sykora.nngui.gui;

import cz.matfyz.sykora.nngui.data.*;
import cz.matfyz.sykora.nngui.exception.DataException;
import cz.matfyz.sykora.nngui.network.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;

/**
 * Dialog pro uen neuronov st pomoc dat natench ze souboru.
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class TrainNetworkDialog extends javax.swing.JDialog {
    /**
     * Promnn urujc, jestli je vpoet aktivn (tj. jsou zpracovvny
     * zznamy ze soubor. Pouv se k pedasnmu zastaven zpracovn dat
     * (b v samostatnm vlkn) pomoc tlatka stop.
     *  @see #onStartTrainingNetwork
     *  @see #onStopTraining
     */
    private volatile boolean calculationActive;
    /**
     * Prv uen neuronov s.
     */
    private Network network;
    /**
     * Komponenta obslujujc dialog pro vybrn soubor, ze kterch je
     * neuronov s uen.
     *  @see onSelectDesiredOutputFile
     *  @see onSelectInputSamplesFile
     */
    private JFileChooser inputFileChooser;
    
    /**
     * Konstruktor. Incializuje <i>inputFileChooser</i>.
     *  @see #inputFileChooser
     */
    public TrainNetworkDialog(Network _network, java.awt.Frame _parent) {
        super(_parent, true);
        initComponents();
        
        inputFileChooser = new JFileChooser();
        inputFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        
        calculationActive = false;
        enableControls(true);
        network = _network;
    }
    
    /**
     * Povol/zake pouit ovldacch prvk. Vyuv se bhem vpotu, kdy
     * nen mon pouvat jin ovldac prvky, ne tlatko <i>Stop</i>, kter
     * naopak nelze pout, pokud vpoet neb.
     *  @param _enable nastavuje, jestli se maj ovldac prvky povolit, nebo
     *              zakzat
     */
    protected void enableControls(boolean _enable) {
        inputSamplesFileEditor.setEnabled(_enable);
        selectInputSamplesFileButton.setEnabled(_enable);
        desiredOutputFileEditor.setEnabled(_enable);
        selectDesiredOutputFileButton.setEnabled(_enable);
        useAllSamplesButton.setEnabled(_enable);
        randomOrderSamplesButton.setEnabled(_enable);
        numberOfIterationsEditor.setEnabled(_enable);
        startTrainingButton.setEnabled(_enable);
        stopTraining.setEnabled(!_enable);
        setDefaultCloseOperation(_enable ? JFrame.DISPOSE_ON_CLOSE : JFrame.DO_NOTHING_ON_CLOSE);
    }
    
    /**
     * Inicializace ovldacch prvk formule. Automaticky generovno NetBeans
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        javax.swing.JLabel labelInputSamples;
        javax.swing.JLabel learningSpeedLabel;
        javax.swing.JLabel numberOfIterationsLabel;
        javax.swing.JLabel progressCaptionLabel;

        buttonGroup2 = new javax.swing.ButtonGroup();
        labelInputSamples = new javax.swing.JLabel();
        labelDesiredOutput = new javax.swing.JLabel();
        inputSamplesFileEditor = new javax.swing.JTextField();
        desiredOutputFileEditor = new javax.swing.JTextField();
        selectInputSamplesFileButton = new javax.swing.JButton();
        selectDesiredOutputFileButton = new javax.swing.JButton();
        useAllSamplesButton = new javax.swing.JRadioButton();
        randomOrderSamplesButton = new javax.swing.JRadioButton();
        numberOfIterationsEditor = new javax.swing.JSpinner();
        numberOfIterationsLabel = new javax.swing.JLabel();
        progressCaptionLabel = new javax.swing.JLabel();
        progressLabel = new javax.swing.JLabel();
        startTrainingButton = new javax.swing.JButton();
        learningSpeedLabel = new javax.swing.JLabel();
        learningSpeedEditor = new javax.swing.JTextField();
        stopTraining = new javax.swing.JButton();

        getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Train network");
        setResizable(false);
        labelInputSamples.setText("Input samples:");
        getContentPane().add(labelInputSamples, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 40, 110, 20));

        labelDesiredOutput.setText("Desired output:");
        getContentPane().add(labelDesiredOutput, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 70, 110, 20));

        getContentPane().add(inputSamplesFileEditor, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 40, 200, -1));

        getContentPane().add(desiredOutputFileEditor, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 70, 200, -1));

        selectInputSamplesFileButton.setText("jButton1");
        selectInputSamplesFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSelectInputSamplesFile(evt);
            }
        });

        getContentPane().add(selectInputSamplesFileButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 40, 20, 20));

        selectDesiredOutputFileButton.setText("jButton2");
        selectDesiredOutputFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSelectDesiredOutputFile(evt);
            }
        });

        getContentPane().add(selectDesiredOutputFileButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 70, 20, 20));

        buttonGroup2.add(useAllSamplesButton);
        useAllSamplesButton.setSelected(true);
        useAllSamplesButton.setText("Train on all samples in the order they come");
        getContentPane().add(useAllSamplesButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 100, 330, -1));

        buttonGroup2.add(randomOrderSamplesButton);
        randomOrderSamplesButton.setText("Train on samples in random order");
        getContentPane().add(randomOrderSamplesButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 130, -1, -1));

        getContentPane().add(numberOfIterationsEditor, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 160, 180, -1));

        numberOfIterationsLabel.setText("Number of iterations");
        getContentPane().add(numberOfIterationsLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 160, 150, 20));

        progressCaptionLabel.setText("Progress:");
        getContentPane().add(progressCaptionLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 200, 80, 20));

        progressLabel.setText("(not started yet)");
        progressLabel.setBorder(new javax.swing.border.EtchedBorder());
        getContentPane().add(progressLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 200, 220, 20));

        startTrainingButton.setText("Start");
        startTrainingButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onStartTrainingNetwork(evt);
            }
        });

        getContentPane().add(startTrainingButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(170, 230, 80, 30));

        learningSpeedLabel.setText("Learning speed:");
        getContentPane().add(learningSpeedLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 10, 110, 20));

        learningSpeedEditor.setText("0.001");
        getContentPane().add(learningSpeedEditor, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 10, 220, 20));

        stopTraining.setText("Stop");
        stopTraining.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onStopTraining(evt);
            }
        });

        getContentPane().add(stopTraining, new org.netbeans.lib.awtextra.AbsoluteConstraints(260, 230, 80, 30));

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-353)/2, (screenSize.height-297)/2, 353, 297);
    }
    // </editor-fold>//GEN-END:initComponents

    /**
     * Obsluha stisknut tlatka <i>Stop</i>.
     *  @param evt informace o udlosti GUI
     *  @see #calculationActive
     */
    private void onStopTraining(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onStopTraining
        calculationActive = false;
    }//GEN-LAST:event_onStopTraining

    /**
     * Obsluha stisknut tlatka start. Provede kontrolu zadanch paramtr pro
     * uen a spust nov vlkno, ve kterm b algoritmus zptnho en.
     * Toto vlkno skon ve chvli, kdy je zpracovan zadan mnostv vzork,
     * nebo kdy je vpoet peruen stisknutm tlatka <i>Stop</i>
     *  @param evt informace o udlosti GUI
     *  @see #calculationActive
     *  @see #network
     */
    private void onStartTrainingNetwork(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onStartTrainingNetwork
        DataProvider input = null;
        DataProvider desired_output = null;
        
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        
        String learningSpeedText = learningSpeedEditor.getText();
        double learningSpeed = 0.001;
        try {
            learningSpeed = Double.parseDouble(learningSpeedText);
        }
        catch(NumberFormatException nf_err) {
            JOptionPane.showMessageDialog(this, "Learning speed is not a valid number",
                        "Network training", JOptionPane.ERROR_MESSAGE);
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        
        try {
            File input_file = new File(inputSamplesFileEditor.getText());
            if(!input_file.canRead()) {
                JOptionPane.showMessageDialog(this, "Input samples file does not exist or could not be read",
                        "Network training", JOptionPane.WARNING_MESSAGE);
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                return;
            }
            input = new TextStreamProvider(input_file);
        }
        catch(FileNotFoundException f_err) {
            JOptionPane.showMessageDialog(this, "File with input samples could not be open for reading: " + f_err.getLocalizedMessage(),
                        "Network training", JOptionPane.ERROR_MESSAGE);
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        try {
            File desired_output_file = new File(desiredOutputFileEditor.getText());
            if(!desired_output_file.canRead()) {
                JOptionPane.showMessageDialog(this, "Desired output file does not exist or could not be read",
                        "Network training", JOptionPane.WARNING_MESSAGE);
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                return;
            }
            desired_output = new TextStreamProvider(desired_output_file);
        }
        catch(FileNotFoundException f_err) {
            JOptionPane.showMessageDialog(this, "File with desired output could not be open for reading: " + f_err.getLocalizedMessage(),
                        "Network training", JOptionPane.ERROR_MESSAGE);
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        
        if(randomOrderSamplesButton.isSelected()) {
            RandomizedAdaptionProvider random = new RandomizedAdaptionProvider(input, desired_output);
            input = random;
            desired_output = random.getDesiredOutputProvider();
        }
        
        try {
            network.setInputProvider(input);
            network.setDesiredOutputProvider(desired_output);
        }
        catch(DataException d_err) {
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            return;
        }
        
        // vlastnosti vypoctu jsou nastaveny jako final tak, aby k nim bylo
        // mozne pristupovat z pocitajiciho vlakna
        final int iterations = randomOrderSamplesButton.isSelected() ?
                    ((Integer)numberOfIterationsEditor.getValue()).intValue() : -1;
        final AdaptationStrategy strategy = new SimpleBackPropagation(learningSpeed);
        
        // spust vypocet v novem vlakne
        Thread working = new Thread() {
            public void run() {
                int iters = iterations;
                
                calculationActive = true;
                enableControls(false);
                while(iters != 0 && calculationActive) {
                    try {
                        if(!network.adaptSingle(strategy))
                            break;
                    }
                    catch(DataException d_err) {
                        break;
                    }
                    if(iters > 0) {
                        iters--;
                        progressLabel.setText("" + iters + " iterations left");
                    }
                }
                calculationActive = false;
                enableControls(true);
            }
        };
        working.start();
        
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }//GEN-LAST:event_onStartTrainingNetwork

    /**
     * Obsluha tlatka pro vbr souboru s poadovanmi vstupy st
     *  @param evt informace o udlosti v GUI
     *  @see #inputFileChooser
     */
    private void onSelectDesiredOutputFile(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onSelectDesiredOutputFile
        int ret = inputFileChooser.showOpenDialog(this);
        if(JFileChooser.APPROVE_OPTION != ret)
            return;
        
        File selected_file = inputFileChooser.getSelectedFile();
        if(selected_file.exists())
            desiredOutputFileEditor.setText(selected_file.getAbsolutePath());
    }//GEN-LAST:event_onSelectDesiredOutputFile

    /**
     * Obsluha tlatka pro vbr souboru se vstupnmi vzorky
     *  @param evt informace o udlosti v GUI
     *  @see #inputFileChooser
     */
    private void onSelectInputSamplesFile(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onSelectInputSamplesFile
        int ret = inputFileChooser.showOpenDialog(this);
        if(JFileChooser.APPROVE_OPTION != ret)
            return;
        
        File selected_file = inputFileChooser.getSelectedFile();
        if(selected_file.exists())
            inputSamplesFileEditor.setText(selected_file.getAbsolutePath());
    }//GEN-LAST:event_onSelectInputSamplesFile
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup2;
    private javax.swing.JTextField desiredOutputFileEditor;
    private javax.swing.JTextField inputSamplesFileEditor;
    private javax.swing.JLabel labelDesiredOutput;
    private javax.swing.JTextField learningSpeedEditor;
    private javax.swing.JSpinner numberOfIterationsEditor;
    private javax.swing.JLabel progressLabel;
    private javax.swing.JRadioButton randomOrderSamplesButton;
    private javax.swing.JButton selectDesiredOutputFileButton;
    private javax.swing.JButton selectInputSamplesFileButton;
    private javax.swing.JButton startTrainingButton;
    private javax.swing.JButton stopTraining;
    private javax.swing.JRadioButton useAllSamplesButton;
    // End of variables declaration//GEN-END:variables
    
}
