/*
 * NetworkModel.java
 *
 * Created on 29. kvten 2005, 22:23
 */

package cz.matfyz.sykora.nngui.network;

import cz.matfyz.sykora.nngui.exception.InvalidDataFormatException;
import org.w3c.dom.DOMException;

/**
 * Model st uruje jej chovn pomoc zadan penosov funkce. Tato funkce
 * ovlivuje jednak chovn st pi vpotu a jednak vpoty pi jejm uen
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public interface NetworkModel {
    /**
     * Zjitn jmna (jednoznanho identifiktoru) modelu pro pouit nap.
     * pi exportu st do XML.
     *  @return jmno jednoznan urujc typ modelu st
     */
    public String getModelName();
    /**
     * Export specifickch parametr konkrtn instance modelu st. Parametry
     * by se mly uloit ve form synovskch XML uzl dodanho elementu
     * <i>_target</i>. Tato metoda je volna pi ukldn st do XML.
     *  @param _target element reprezentujc model st, do tohoto elementu by
     *              mly bt vloen uzly uchovvajc hodnotu jednotlivch
     *              parametr modelu.
     */
    public void exportModelParameters(org.w3c.dom.Element _target) throws DOMException;
    /**
     * Naten parametr modelu konkrtni instanc modelu st. Naten
     * parametry by mly odpovdat parametrm ukldanm metodou
     * <i>exportModelParameters</i>
     *  @param _source XML element, ze kterho jsou parametry natny
     *  @throws DOMException pi chyb pi prci s DOM stromem
     *  @throws IvalidDataFormatException pokud formt dat neodpovd formtu
     *              pouitmu v tomto modelu.
     */
    public void loadModelParameters(org.w3c.dom.Element _source) throws DOMException,
                                                                        InvalidDataFormatException;
    /**
     * Penosov funkce. Uruje jakm zpsobem se transformuje hodnta aktivace
     * neuronu na vstupn hodnotu.
     *  @param _excitation aktivace neuronu (souet vench vstupnch hodnot
     *              a prahu neuronu)
     *  @return vstupn hodnota neuronu
     */
    public double transitionFunction(double _excitation);
    /**
     * Prvn derivace penosov funkce podle aktivace. Pouv se pi vpotu
     * zmny vah pi uen st
     *  @param _excitation aktivace neuronu (souet vench vstupnch hodnot
     *              a prahu neuronu)
     *  @return prvn derivace penosov funkce podle aktivace pro zadanou
     *              hodnotu aktivace
     */
    public double transitionFunctionDerivation(double _excitation);
}
