/*
 * NeuronRow.java
 *
 * Created on 4. kvten 2005, 15:35
 */

package cz.matfyz.sykora.nngui.network;

/**
 * Reprezentace jedn vrstvy neuron, zapouzduje nkter innosti specific
 * prv pro dky neuron
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class NeuronRow {
    /**
     * Pole s neurony obsaenmi v tto vtstv. Toto pole me bt pedvno dl
     * pi nkterch operacch s neurony (nap. pi vytven spoj)
     */
    private Neuron[] neurons;
    /**
     * Zptn odkaz na neuronovu s vlastnc tuto adu neuron
     */
    private Network network;
    
    /**
     * Adaptace pro jednu adu neuron.
     * Pedv jednotliv neurony adaptan strategii, zmny vah a dal pravy
     * neuron provd ona.
     *  @param _strategy adaptan strategie, kter m bt pi adaptaci pouita
     *  @see AdaptationStrategy#adaptNeuron
     */
    public final void adapt(AdaptationStrategy _strategy) {
        assert(_strategy != null);
        
        for (int i = 0; i < neurons.length; i++)
            _strategy.adaptNeuron(neurons[i]);
    }
    
    /**
     * Vpoet vstupnch hodnot pro vechny neurony v ad
     *  @see #neurons
     *  @see Neuron#calculate
     */
    public final void calculate() {
        assert(neurons != null);
        
        for (int i = 0; i < neurons.length; i++)
            neurons[i].calculate();
    }
    
    /**
     * Pstup k neuronu z tto vrstvy na zadan pozici
     *  @param _index index (pozice) neuronu, ke ktermu pistupujeme
     *  @return neuron na pozici <i>_index</i>
     *  @see #neurons
     *  @see #getSize
     *  @throws ArrayIndexOutOfBounds
     */
    public final Neuron getNeuron(int _index) {
        assert(neurons != null);
        return neurons[_index];
    }
    
    /**
     * Zjitn velikosti tto vrstvy neouron
     *  @see #neurons
     */
    public final int getSize() {
        assert(neurons != null);
        return neurons.length;
    }

    /**
     * Inicializuje spojen mezi neurony tto a pedchoz vrstvy, alokuje zde
     * pole pro vhy jednotlivch neuron a napluje je nhodnmi malmi
     * hodnotami.
     * Nastavuje seznam neuron pedchoz vrstvy na zklad informac o
     * pedchoz vrstv. Toto pole se bhem vpotu, ani bhem uen st
     * nemn, je proto sdleno vemi neurony (navc se pouv pmo pole, ve
     * kterm m seznam svch neuron uloen objekt reprezentujc pedchoz
     * vrstvu.
     *  @param _previous seznam neuron pedchoz vrstvy
     *  @see Neuron#setPredecessors
     *  @see Neuron#setWeights
     *  @see Neuron#initializeRandomWeights
     */
    protected final void initializeConnections(NeuronRow _previous) {
        assert(_previous != null);
        assert(neurons != null);
        
        for (int i = 0; i < neurons.length; i++) {
            double[] weights = new double[_previous.getSize()];
            neurons[i].setPredecessors(_previous.neurons);
            neurons[i].setWeights(weights);
            neurons[i].initializeRandomWeights();
        }
    }
    
    /**
     * Zpracovn rozdl mezi skutenm a oekvanm vstupem.
     * Provede zpracovn rozdl mezi poadovanm a oekvanm vstupem u
     * jednotlivch neuron.
     *  @see Neuron#processDifference
     */
    protected final void processDifferences() {
        assert(neurons != null);
        
        for (int i = 0; i < neurons.length; i++)
            neurons[i].processDifference();
    }
    
    /**
     * Reset rozdl mezi oekvanm a skutenm vstupem pro adu neuron na 0.
     *  @see Neuron#setDifference
     */
    protected final void resetDifferences() {
        assert(neurons != null);
        
        for (int i = 0; i < neurons.length; i++)
            neurons[i].setDifference(0.0);
    }
    
    /**
     * Konstruktor. Vytvo neurony ve vrstv, neprovd ale adnou inicializaci
     * spoj mezi neurony
     *  @param _network s, do kter vrstvu inicializujeme
     *  @param _count poet neuron ve vrstv
     */
    public NeuronRow(Network _network, int _count) {
        assert(_count > 0);
        network = _network;
        
        neurons = new Neuron[_count];
        for (int i = 0; i < neurons.length; i++)
            neurons[i] = new Neuron(network);
    }
    
}
