/*
 * SigmoidModel.java
 *
 * Created on 31. kvten 2005, 21:59
 */

package cz.matfyz.sykora.nngui.network;

/**
 * Model st s sigmoidou (1 / (1 + e^-t)) jako penosovou funkc
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class SigmoidModel implements NetworkModel {
    /**
     * Nzev modelu st (pouiteln pro jeho identifikaci pi ukladn a
     * natn st ze souboru
     *  @see NetworkBuilder
     */
    public static final String  MODEL_NAME = "Sigmoid";
    
    /**
     * Export parametr tto instance modelu st. Tato funkce nedl nic,
     * protoe SigmoidModel dn parametry nepouv.
     *  @param _target XML element, do kterho jsou jako synovsk uzly ukldny
     *              parametry modelu
     */
    public void exportModelParameters(org.w3c.dom.Element _target) {
        if(_target == null)
            throw new NullPointerException("_target must not be null");
    }
    
    /**
     * Naten parametr tto instance modelu st. Tato funkce nedl nic,
     * protoe SigmoidModel dn parametry nepouv.
     *  @param _source XML element, ze kterho jsou parametry natny
     */
    public void loadModelParameters(org.w3c.dom.Element _source) {
        if(_source == null)
            throw new NullPointerException("_source must not be null");
    }
    
    /**
     * Zjitn jmna (jednoznanho identifiktoru) modelu pro pouit nap.
     * pi exportu st do XML.
     *  @return jmno jenoznan urujc model st
     *  @see #MODEL_NAME
     */
    public final String getModelName() {
        return MODEL_NAME;
    }
    
    /**
     * Vpoet penosov funkce pro zadanou aktivan hodnotu neuronu.
     * Pro sigmoidu je to (1 / (1 + e^-t)).
     *  @param _excitation aktivace neuronu, pro kter penosovou funkci
     *              potme
     *  @return hodnota penosov funkce pro zadanou aktivaci neuronu
     */
    public double transitionFunction(double _excitation) {
        return 1 / (1 + Math.exp(-_excitation));
    }
    
    /**
     * Vpoet prvn derivace penosov funkce pro zadanou aktivan hodnotu
     * neuronu.
     * Pro sigmoidu je to t * (1 - t), kde t je hodnota penosov funkce pro
     * zadanou aktivaci neuronu
     *  @param _excitation aktivace neuronu, pro kter penosovou funkci
     *              potme
     *  @return hodnota prvn derivace penosov funkce pro zadanou aktivaci
     *              neuronu
     */
    public double transitionFunctionDerivation(double _excitation) {
        double out = transitionFunction(_excitation);
        return out * (1 - out);
    }
}
