/*
 * SimpleBackPropagation.java
 *
 * Created on 9. kvten 2005, 23:06
 */

package cz.matfyz.sykora.nngui.network;

import javax.swing.*;
import javax.swing.event.*;

/**
 * Implementace jednoduch back-propagation adaptace s nastavitelnm parametrem
 * uen, ale bez jakchkoliv dalch vylepen
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class SimpleBackPropagation implements AdaptationStrategy {
    
    /**
     * Rychlost uen neuronov st.
     * Koeficient ovlivujc velikost zmn provdnch v neuronov sti v
     * jednom kroce adaptace.
     *  @see #adaptNeuron
     */
    private double learningSpeed;
    
    private JPanel settings;
    
    public void adaptNeuron(Neuron _neuron) {
        assert(_neuron != null);
        
        double[] weights = _neuron.getWeights();
        double[] weight_delta = _neuron.getWeightDelta();
        
        assert(weights != null);
        assert(weight_delta != null);
        
        for(int i = 0; i < weights.length; i++) {
            // aktualizuj vahy a prah site podle akumulovanych zmen a koeficientu
            // rychlosti uceni
            weights[i] += learningSpeed * weight_delta[i];
            
            // vynuluj akumulovane zmeny vah a koeficientu uceni
            weight_delta[i] = 0.0;
        }
        _neuron.setThreshold(_neuron.getThreshold() + learningSpeed * _neuron.getThresholdDelta());
        _neuron.setThresholdDelta(0.0);
    }
    
    /**
     * Zjitn rychlosti uen neuronov st
     *  @return aktuln rychlost uen neuronov st
     */
    public double getLearningSpeed() {
        return learningSpeed;
    }
    
    public JPanel getSettingsPanel() {
        if(settings == null) {
            settings = new JPanel();
            
            JSpinner learnSpd = new JSpinner();
            learnSpd.addChangeListener(new ChangeListener() {
                                            public void stateChanged(ChangeEvent evt) {
                                                JSpinner spin = (JSpinner)evt.getSource();
                                                learningSpeed = ((Number)spin.getValue()).doubleValue();
                                            }
                                        });
            JLabel learnSpdLbl = new JLabel("Rychlost uen:");
            learnSpdLbl.setLabelFor(learnSpd);
            settings.add(learnSpdLbl);
            settings.add(learnSpd);
        }
        
        return settings;
    }
    
    /**
     * Nastaven ryhclosti uen neuronov st
     *  @param _learning_speed nov koeficient uen neuronov st
     *  @see #learningSpeed
     */
    public void setLearningSpeed(double _learning_speed) {
        learningSpeed = _learning_speed;
    }
    
    /**
     * Konstruktor.
     *  @param _learning_speed rychlost uen neuronov st
     */
    public SimpleBackPropagation(double _learning_speed) {
        learningSpeed = _learning_speed;
    }
}
