/*
 * NetworkBuilderTest.java
 * JUnit based test
 *
 * Created on 22. kvten 2005, 13:50
 */

package cz.matfyz.sykora.nngui.network;

import junit.framework.*;

/**
 *
 * @author sykorao
 */
public class NetworkBuilderTest extends TestCase {
    
    protected NetworkBuilder builder;
    
    public NetworkBuilderTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
        builder = new NetworkBuilder();
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(NetworkBuilderTest.class);
        
        return suite;
    }

    /**
     * Test pro metodu <i>createNetwork</i>. Naped zkou podstrit chybn
     * zadn st, jestli projde a pokud projde, zkus vytvoit tak njakou
     * s podle sprvanho zadn
     */
    public void testCreateNetwork() {
        System.out.println("testCreateNetwork");
        
        boolean hadException = false;
        Network net = null;
        
        // test, jestli metoda selze spravnym zpusobem, pokud predame misto pole
        // s informacemi o architekture site null
        System.out.println("\t- createNetwork(null)");
        try {
            net = builder.createNetwork(null,null);
        }
        catch(NullPointerException e) {
            hadException = true;
        }
        catch(Throwable t) {
            t.printStackTrace();
            fail("Unexpected exception occured: " + t.toString());
        }
        if(!hadException)
            fail("No exception occured on invalid input");
        
        // test, jestli metoda selze spravnym zpusobem, pokud predame prazdne
        // pole (existujici pole, ktere ale ma 0 prvku)
        hadException = false;
        System.out.println("\t- createNetwork([empty array])");
        try {
            int[] arch = {};
            net = builder.createNetwork(arch, new SigmoidModel());
        }
        catch(IllegalArgumentException e) {
            hadException = true;
        }
        catch(Throwable t) {
            t.printStackTrace();
            fail("Unexpected exception occured: " + t.toString());
        }
        
        System.out.println("\t- createNetwork({3, 2, 3})");
        try {
            int[] arch = { 3, 2, 3 };
            net = builder.createNetwork(arch, new SigmoidModel());
            
            int[] created = net.getArchitecture();
            
            if(arch.length != created.length)
                fail("Numbers of rows differ");
            for(int i = 0; i < arch.length; i++)
                if(arch[i] != created[i])
                    fail("Numbers of neurons on row " + i + "differ");
        }
        catch(Throwable t) {
            t.printStackTrace();
            fail("Unexpected exception occured: " + t.toString());
        }
    }
}
