/*
 * NetworkTest.java
 * JUnit based test
 *
 * Created on 22. kvten 2005, 21:36
 */

package cz.matfyz.sykora.nngui.network;

import java.io.FileNotFoundException;
import junit.framework.*;
import cz.matfyz.sykora.nngui.exception.NoDataSourceException;
import java.util.*;
import cz.matfyz.sykora.nngui.data.*;
import cz.matfyz.sykora.nngui.exception.*;

/**
 * Komplexn test pro prci s neuronovou st. sten stav na funknosti
 * NetworkBuilderu a spolh na to, e jeho testy probhly spn
 *  @author Ondej Skora [ondrasej@centrum.cz]
 */
public class NetworkTest extends TestCase {
    
    public NetworkTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(NetworkTest.class);
        
        return suite;
    }

    public void testSimpleBackProp() {
        NetworkBuilder builder = new NetworkBuilder();
        
        Network or_net = builder.createNetwork(new int[] { 2, 1 }, new SigmoidModel());
        if(or_net == null)
            fail("network creation has failed");
        
        DataProvider input_provider = null;
        DataProvider desired_output_provider = null;
        DataConsumer output_consumer = null;
        
        
        try {
            input_provider = new TextStreamProvider("./and_input.txt");
            desired_output_provider = new TextStreamProvider("./and_expected_output.txt");
            
            DataProvider randomized = new RandomizedAdaptionProvider(input_provider,desired_output_provider);
            
            input_provider.close();
            desired_output_provider.close();
            
            desired_output_provider = ((RandomizedAdaptionProvider)randomized).getDesiredOutputProvider();
            input_provider = randomized;
            
            output_consumer = new TextStreamConsumer("./and_output.txt");
        }
        catch(FileNotFoundException fnf) {
            fail("Input file was not found: " + fnf.getMessage());
        }
       
        try {
            or_net.setInputProvider(input_provider);
            or_net.setDesiredOutputProvider(desired_output_provider);
            or_net.setOutputConsumer(output_consumer);
        }
        catch (DataException dt) {
            fail("Could not set providers and consumers: " + dt.getMessage());
        }
        
        AdaptationStrategy backprop = new SimpleBackPropagation(1);
        try {
            int iterations = 1000;
            while(iterations-- > 0 && or_net.adapt(backprop, 10) == 10) {
                System.out.print(input_provider.getValue(0));
                System.out.print(" ");
                System.out.print(input_provider.getValue(1));
                System.out.print(" => ");
                System.out.print(or_net.getNeuron(1, 0).getOutput());
                System.out.println();
            }
        }
        catch (DataException dt) {
            fail("Backpropagation failed: " + dt.getMessage());
        }
        
        input_provider.close();
        desired_output_provider.close();
        output_consumer.close();
    }
}
